package org.saev.deob;

import java.util.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.URI;
import org.jdom.*;
import org.jdom.input.*;

public class TransformerFactory {
	private static Map<String, Transformer> classMap;
	
	static {
		classMap = new HashMap<String, Transformer>();
		File conf = null;
		ClassLoader cl = ClassLoader.getSystemClassLoader();
		String className = null;
		
		try {
			Document d;
			conf = new File(".", "config.xml");
			
			if(conf.exists()) {
				d = new SAXBuilder().build(conf);
			} else {
				System.out.println("error: could not find config.xml, trying internal copy");
				d = new SAXBuilder().build(cl.getResource("config.xml"));
			}
			Element root = d.getRootElement();
			List<Element> elements = root.getChildren();

			for(Element e : elements) {
				className = e.getAttributeValue("class");
				Transformer t = (Transformer) cl.loadClass(className).newInstance();
				String name = t.getName();
				//System.out.println("found transformer " + name);
				classMap.put(name, t);
			}
		} catch(FileNotFoundException e) {
			System.err.println("fatal: could not find config.xml");
			System.exit(1);
		} catch(JDOMException e) {
			System.err.println("fatal: could not parse config.xml");
			//e.printStackTrace();
			System.exit(1);
		} catch(Exception e) {
			System.err.println("error: could not load " + className);
			//e.printStackTrace();
		}
	};
	
	public static Transformer[] getTransformers() {
		return classMap.values().toArray(new Transformer[] {});
	}

	public static Transformer getTransformer(String transName) {
		return classMap.get(transName);
	}
}