package org.saev.deob.trans.pred;

import org.saev.deob.*;
import org.saev.deob.util.*;
import java.util.*;
import org.apache.bcel.classfile.*;
import org.apache.bcel.generic.*;
import org.apache.bcel.util.*;

public class OpaqueRemover extends ApplicationTransformer {
	public static final boolean DEBUG = false;
	private ControlFinder finder;
	
	public OpaqueRemover() {
		super();
	}

	public void transform(Application app) {
		finder = new CPControlFinder(app);
		super.transform(app);
	}

	protected Method internalTransform(MethodGen mg) {
		
		if(DEBUG) {
			System.out.println("\ntransforming method: " + new MethodID(mg));
		}
		final InstructionList il = mg.getInstructionList();
		final ConstantPoolGen cpg = mg.getConstantPool();
		Iterator<InstructionHandle[]> matchIt =
				new InstructionFinder(il).search("(GETSTATIC | ILOAD) (ISTORE | (IFEQ | IFNE))" +
						"(((IINC ILOAD) | ((ILOAD IFEQ)? ICONST GOTO ICONST)) PUTSTATIC)?",
				new InstructionFinder.CodeConstraint() {
			
			public boolean checkCode(InstructionHandle[] match) {
				Instruction first = match[0].getInstruction();
				if(match.length < 3) {
					
					if(first instanceof GETSTATIC) {
						FieldInstruction fi = (FieldInstruction) first;
						return finder.containsField(new FieldID(fi.getName(cpg), fi.getSignature(cpg), fi.getClassName(cpg)));
					} else if(first instanceof ILOAD) {
						LocalVariableInstruction lvi = (LocalVariableInstruction) first;
						
						for(Iterator<InstructionHandle[]> it = new InstructionFinder(il).search("GETSTATIC ISTORE"); it.hasNext(); ) {
							InstructionHandle[] ihs = it.next();
							FieldInstruction fi = (FieldInstruction) ihs[0].getInstruction();
							
							if(finder.containsField(new FieldID(fi.getName(cpg), fi.getSignature(cpg), fi.getClassName(cpg)))) {
								return lvi.getIndex() == ((LocalVariableInstruction) ihs[1].getInstruction()).getIndex();
							}
						}
					}
				} else if(first instanceof GETSTATIC) {
					FieldInstruction fi = (FieldInstruction) first;
					return new FieldID(fi.getName(cpg), fi.getSignature(cpg), fi.getClassName(cpg)).equals(finder.getControlField());
				}
				return false;
			}
		});
		
		while(matchIt.hasNext()) {
			InstructionHandle[] match = matchIt.next();
			
			if(DEBUG) {
				System.out.println("\nmatch = " + Arrays.asList(match));
			}
			InstructionHandle first = match[0];
			InstructionHandle last = match[match.length - 1];
			InstructionHandle next;
			
			if((next = last.getNext()) == null) {
				break;
			}
			
			if(match.length < 3) {
				
				if(last.getInstruction() instanceof ISTORE) {
					
					try {
						
						if(DEBUG) {
							System.out.println("removing = " + Arrays.asList(match));
						}
						il.delete(first, last);
					} catch(TargetLostException e) {
						InstructionHandle[] targets = e.getTargets();
						
						for(int j = 0; j < targets.length; j++) {
							InstructionTargeter[] targeters = targets[j].getTargeters();
							
							for(int k = 0; k < targeters.length; k++) {
								targeters[k].updateTarget(targets[j], next);
							}
						}
					}
				} else if(last.getInstruction() instanceof IFEQ) {
					InstructionHandle target = ((BranchHandle) last).getTarget();
					last.setInstruction(new GOTO(target));
					if(DEBUG) {
						System.out.println("last = " + last);
					}
					
					try {
						
						if(DEBUG) {
							System.out.println("removing = " + first);
						}
						il.delete(first);
					} catch(TargetLostException e) {
						InstructionHandle[] targets = e.getTargets();
						
						for(int j = 0; j < targets.length; j++) {
							InstructionTargeter[] targeters = targets[j].getTargeters();
							
							for(int k = 0; k < targeters.length; k++) {
								targeters[k].updateTarget(targets[j], last);
							}
						}
					}
				} else if(last.getInstruction() instanceof IFNE) {
					try {
						
						if(DEBUG) {
							System.out.println("removing = " + Arrays.asList(match));
						}
						il.delete(first, last);
					} catch(TargetLostException e) {
						InstructionHandle[] targets = e.getTargets();
						
						for(int j = 0; j < targets.length; j++) {
							InstructionTargeter[] targeters = targets[j].getTargeters();
							
							for(int k = 0; k < targeters.length; k++) {
								targeters[k].updateTarget(targets[j], next);
							}
						}
					}
				}
			} else {
				
				try {
					
					if(DEBUG) {
						System.out.println("removing = " + Arrays.asList(match));
					}
					il.delete(first, last);
				} catch(TargetLostException e) {
					InstructionHandle[] targets = e.getTargets();
					
					for(int j = 0; j < targets.length; j++) {
						InstructionTargeter[] targeters = targets[j].getTargeters();
						
						for(int k = 0; k < targeters.length; k++) {
							targeters[k].updateTarget(targets[j], next);
						}
					}
				}
			}
		}
		mg.setInstructionList(il);
		mg.setMaxLocals();
		mg.setMaxStack();
		return mg.getMethod();
	}

	public String getDescription() {
		return "does an alright job removing the majority\n" +
			"\tof zelix klassmaster opaque predicates. mainly for\n" +
			"\tuse with the rs clients";
	}

	public String getAuthor() {
		return "saevion <saevion at gmail dot com>";
	}

	public String getName() {
		return "zkmstripper";
	}
}